#include "bb_api.h"
#pragma comment(lib, "bb_api.lib")

#include <iostream>
#include <vector>

/*
This is the same example as iq_example, except we specify a return 
data type of complex shorts and show the conversion to float.
*/

void iqExample16sc()
{
    int handle = -1;
    bbStatus status = bbOpenDevice(&handle);
    if(status != bbNoError) {
        std::cout << bbGetErrorString(status);
        return;
    }

    // Specify 16-bit complex shorts as the data type
    bbConfigureIQDataType(handle, bbDataType16sc);
    // Set center frequency
    bbConfigureIQCenter(handle, 2400.0e6);
    // Set reference level and auto gain/atten
    bbConfigureLevel(handle, -20.0, BB_AUTO_ATTEN);
    bbConfigureGain(handle, BB_AUTO_GAIN);
    // Set a sample rate of 40.0e6 / 2 = 20.0e6 MS/s and bandwidth of 15 MHz
    bbConfigureIQ(handle, 2, 15.0e6);

    // Initialize the device for IQ streaming
    status = bbInitiate(handle, BB_STREAMING, BB_STREAM_IQ);
    if(status != bbNoError) {
        // Handle error
    }

    // Verify bandwidth and sample rate
    double bandwidth;
    int sampleRate;
    bbQueryStreamInfo(handle, 0, &bandwidth, &sampleRate);

    float iqCorrection = 0.0;
    bbGetIQCorrection(handle, &iqCorrection);

    // Allocate memory for BLOCK_SIZE complex values
    const int BLOCK_SIZE = 262144;

    // Allocate memory for both shorts and floats
    std::vector<short> buffer16sc(BLOCK_SIZE * 2);
    std::vector<float> buffer32fc(BLOCK_SIZE * 2);

    // Set up block capture
    bbIQPacket pkt;
    pkt.iqData = &buffer16sc[0];
    pkt.iqCount = BLOCK_SIZE;
    pkt.triggers = 0;
    pkt.triggerCount = 0;
    // Setting purge to true tells the API to return IQ data that has been
    // acquired only after calling the bbGetIQ() function. Setting purge to false
    // will return data contiguous to the data returned from the last call to
    // bbGetIQ().
    pkt.purge = true;

    // Perform capture
    status = bbGetIQ(handle, &pkt);

    // Convert to float, I/Q data is interleaved, apply the same logic to all re/im samples.
    for(int i = 0; i < buffer16sc.size(); i++) {
        // Convert to floats in the range [-1.0, 1.0]
        buffer32fc[i] = (((float)buffer16sc[i]) / 32768.0);
        // Apply correction.
        buffer32fc[i] *= iqCorrection;
    }

    // At this point, BLOCK_SIZE IQ data samples have been retrieved and
    //  stored in the buffer array.
    // bbGetIQ can continue to be called to retrieve more IQ samples.

    bbAbort(handle);
    bbCloseDevice(handle);
}
