/*
 *  Get a VRT Context packet from the Signal Hound SM200A followed by a block of 1000 Signal Data packets
 *
 */

    // Set up device
    int device = -1;
    SmStatus status = smOpenDevice(&device);
    if(status != smNoError) {
        // Could not open SM200A
    }

    // Set IQ parameters
    smSetIQCaptureType(device, smIQStreaming);
    smSetIQCenterFreq(device, 3.0e9);
    smSetIQSampleRate(device, 2);
    smSetIQBandwidth(device, smTrue, 20.0e6);
    smSetRefLevel(device, -20);

    // Set VRt parameters
    smSetVrtPacketSize(device, 16384);

    // Configure
    status = smConfigure(device, smModeIQ); // VRT mode
    if(status != smNoError) {
        // Could not configure SM200A
    }

    // Allocate memory
    uint32_t contextPacketSize;
    status = smGetVrtContextPktSize(device, &contextPacketSize);
    if(status != smNoError) {
        // Could not get context packet size
    }

    int numDataPackets = 1000;
    uint32_t dataPacketSize;
    status = smGetVrtPacketSize(device, &dataPacketSize);
    if(status != smNoError) {
        // Could not get data packet size
    }

    uint32_t wordCount = contextPacketSize + dataPacketSize * numDataPackets;
    uint32_t *words = new uint32_t[wordCount];
    uint32_t *curr = words;

    // Get context packet
    uint32_t actualContextPacketSize;
    status = smGetVrtContextPkt(device, curr, &actualContextPacketSize);
    if(status != smNoError) {
        // Could not get context packet
    }
    if(actualContextPacketSize != contextPacketSize) {
        // Context packet is not the expected size
    }

    // Get data packets
    uint32_t actualDataPacketsBlockSize;
    status = smGetVrtPackets(device, words, &actualDataPacketsBlockSize, numDataPackets, smFalse);
    if(status != smNoError) {
        // Could not get data packets
    }
    if(actualDataPacketsBlockSize != dataPacketSize * numDataPackets) {
        // Block of data packets is not the expected size
    }
