#include "vrt_parser.h"

#include <cassert>

int main()
{
    // Set up device
    int device = -1;
    SmStatus status = smOpenDevice(&device);
    if(status != smNoError) {
        // Could not open spec an
        assert(false, L"Could not open spec an.");
    }

    smSetIQCaptureType(device, smIQStreaming);
    smSetIQCenterFreq(device, 3.0e9);
    smSetIQSampleRate(device, 2);
    smSetIQBandwidth(device, smTrue, 20.0e6);
    smSetRefLevel(device, -20);

    smSetVrtPacketSize(device, 16384);
    smSetVrtStreamID(device, 13);

    status = smConfigure(device, smModeIQ); // VRT mode
    if(status != smNoError) {
        // Could not configure spec an
        assert(false, L"Could not configure spec an.");
    }

    //
    // Get blob of VRT data
    //

    int dataToContextPktRatio = 100;
    int numDataPackets = 3000;
    int numContextPackets = numDataPackets / dataToContextPktRatio;

    // Get context packet size
    uint32_t contextPktWordCount = 0;
    status = smGetVrtContextPktSize(device, &contextPktWordCount);
    if(status != smNoError) {
        // Could not get context packet size
        assert(false, L"Could not get context packet size.");
    }

    // Get data packet size
    uint32_t dataPktWordCount = 0;
    uint16_t samplesPerPacketReturn = 0;
    status = smGetVrtPacketSize(device, &samplesPerPacketReturn, &dataPktWordCount);
    if(status != smNoError) {
        // Could not get data packet size
        assert(false, L"Could not get data packet size.");
    }

    // Allocate memory for blob
    uint32_t wordCount = contextPktWordCount * numContextPackets  +
        dataPktWordCount * numDataPackets;
    uint32_t *words = new uint32_t[wordCount];
    uint32_t *curr = words;

    // Get VRT packets
    for(int i = 0; i < numContextPackets; i++) {
        // Get context packet
        uint32_t actualContextPktWordCount;
        status = smGetVrtContextPkt(device, curr, &actualContextPktWordCount);
        if(status != smNoError) {
            // Could not get context packet
            assert(false, L"Could not get context packet.");
        }
        if(actualContextPktWordCount != contextPktWordCount) {
            // Context packet is not the expected size
            assert(false, L"Context packet is not the expected size.");
        }
        curr += actualContextPktWordCount;

        // Get data packets
        uint32_t actualDataPktWordCount;
        status = smGetVrtPackets(device, curr, &actualDataPktWordCount, dataToContextPktRatio, smFalse);
        if(status != smNoError) {
            // Could not get data packets
            assert(false, L"Could not get data packets.");
        }
        if(actualDataPktWordCount != dataPktWordCount * dataToContextPktRatio) {
            // Data packet is not the expected size
            assert(false, L"Data packet is not the expected size.");
        }
        curr += actualDataPktWordCount;
    }

    // Parse
    VRTParser parser;

    // Set a default reference level in the event that one or more data packets are parsed before the first context packet
    double reflevel;
    smGetRefLevel(device, &reflevel);
    parser.reflevel = reflevel;

    VRTUserDataPkt dataPkt;
    VRTUserContextPkt contextPkt;

    curr = words;
    while(curr - words < wordCount) {
        // Parse next packet
        SmVRTPacketType packetType;
        uint32_t packetSize;
        parser.Peek(curr, &packetType, &packetSize);

        switch(packetType) {
        case smVRTDataPacket:
            curr += parser.ParseDataPacket(curr, packetSize, dataPkt);
            break;
        case smVRTContextPacket:
            curr += parser.ParseContextPacket(curr, packetSize, contextPkt);
            break;
        default:
            // Memory pointed to is not a valid SM200A VRT packet
            break;
        }
    }

    int wordsParsed = curr - words;
}
